package cn.com.qmth.stmms.ms.admin.api;

import java.util.List;
import java.util.stream.Collectors;

import cn.com.qmth.stmms.ms.core.domain.user.MarkRight;
import cn.com.qmth.stmms.ms.core.repository.MarkTaskRepo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.MarkSubjectRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkUserRepo;
import cn.com.qmth.stmms.ms.core.vo.Subject;

/**
 * 
 * @Description: 用户api
 * @author ting.yin
 * @date 2016年10月24日
 */
@RestController
@RequestMapping("api/admin/users")
public class UserApi {

	@Autowired
	private MarkUserRepo markUserRepo;

    @Autowired
    private MarkSubjectRepo markSubjectRepo;

	@Autowired
	private MarkTaskRepo markTaskRepo;
	
	@RequestMapping(value = "{userId}", method = RequestMethod.DELETE)
	public void remove(@PathVariable Long userId) {
		long count = markTaskRepo.countByMarkerId(userId);
		if(count > 0){
			throw new RuntimeException("已分配任务，不允许删除");
		}
		markUserRepo.delete(userId);
	}

	@RequestMapping(value = "{domain}", method = RequestMethod.PUT)
	public void update(@PathVariable MarkUser domain,@RequestBody MarkUser user){
		domain.setName(user.getName());
		domain.setMarkRight(user.getMarkRight());
		markUserRepo.save(domain);
	}

	@RequestMapping(method = RequestMethod.POST)
	public void create(@RequestBody MarkUser user) {
		MarkSubject markSubject = markSubjectRepo.findOne(user.getWorkId() + "-" + user.getSubject().toString());
		if(markSubject == null){
            throw new RuntimeException("该科目不存在");
        }
		if(user.getRole() == Role.MARKER && user.getMarkRight() == MarkRight.ALLOW_LEVELING && !markSubject.getStage().equals(MarkStage.INIT)){
			throw new RuntimeException("请进入分档阶段前创建");
		}
		MarkUser markUser = markUserRepo.findByLoginName(user.getWorkId()+"-"+user.getLoginName());
		if(markUser != null){
            throw new RuntimeException("该登录名已经存在");
        }
		user.setLoginName(user.getWorkId()+"-"+user.getLoginName());
		markUserRepo.save(user);
	}

    @RequestMapping(method = RequestMethod.GET)
    public List<MarkUser> list(@RequestParam Long workId,
							   @RequestParam Subject subject,
							   @RequestParam(required = false ) Role role){
    	if(role!= null){
    		return markUserRepo.findByWorkIdAndRole(workId,role).stream().filter( u -> u.getSubject() == subject)
					.collect(Collectors.toList());
    	}else{
    		return markUserRepo.findByWorkId(workId).stream().filter( u -> u.getSubject() == subject)
					.collect(Collectors.toList());
    	}
    }
	

	
}
