package cn.com.qmth.stmms.ms.marking.api;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import cn.com.qmth.stmms.ms.marking.assembler.LevelStatAssembler;
import cn.com.qmth.stmms.ms.marking.assembler.MarkerAssembler;
import cn.com.qmth.stmms.ms.marking.assembler.QuestionStatAssembler;
import cn.com.qmth.stmms.ms.marking.dto.LevelStatDTO;
import cn.com.qmth.stmms.ms.marking.dto.MarkerDTO;
import cn.com.qmth.stmms.ms.marking.dto.QuestionStatDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import cn.com.qmth.stmms.ms.core.domain.Level;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.user.MarkRight;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.LevelRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkSubjectRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkTaskRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkUserRepo;
import cn.com.qmth.stmms.ms.core.vo.Subject;

/**
 * 评卷中，评卷员的相关数据api
 * Created by zhengmin on 2016/10/14.
 */
@RestController
@RequestMapping("api/markers")
public class MakrerApi {

    @Autowired
    private MarkUserRepo markUserRepo;
    
    @Autowired
    private MarkTaskRepo markTaskRepo;

	@Autowired
	private MarkerAssembler markerAssembler;

    @Autowired
    private QuestionStatAssembler questionStatAssembler;
    
    @Autowired
    private MarkSubjectRepo markSubjectRepo;
    
    @Autowired
    private LevelRepo levelRepo;

    @Autowired
    private LevelStatAssembler levelStatAssembler;

    /**
     * 评卷员信息
     * @param markerId 评卷员用户id
     * @return
     */
	@RequestMapping(value = "{markerId}",method = RequestMethod.GET)
	public MarkerDTO getMarker(@PathVariable Long markerId){
		MarkUser marker = markUserRepo.findOne(markerId);
		return markerAssembler.toDTO(marker);
	}

    /**
     * 评卷员列表
     * @param workId 评卷工作id
     * @param subject 科目
     * @param markStage 允许参与的阶段
     * @return
     */
    @RequestMapping(method = RequestMethod.GET)
    public List<MarkUser> list(@RequestParam Long workId,
                               @RequestParam Subject subject,
                               @RequestParam MarkStage markStage){
    	List<MarkUser>  markUsers = markUserRepo.findByWorkIdAndSubjectAndRole(workId,subject, Role.MARKER);
    	if(MarkStage.LEVEL == markStage ){
    		markUsers = markUsers.stream().filter(m -> m.getMarkRight() == MarkRight.ALLOW_LEVELING||m.getMarkRight() == MarkRight.ALLOW_ALL).collect(Collectors.toList());
    	}else if(MarkStage.SCORE == markStage){
    		markUsers = markUsers.stream().filter(m -> m.getMarkRight() == MarkRight.ALLOW_SCORING||m.getMarkRight() == MarkRight.ALLOW_ALL).collect(Collectors.toList());
    	}
        return markUsers;
    }

    /**
     * 评卷员分档数量及占比数据
     * @param marker 评卷员用户id
     * @param questionId 试题id
     * @return
     */
    @RequestMapping(value = "{marker}/stat/levels",method = RequestMethod.GET)
    public List<LevelStatDTO> levelStats(@PathVariable MarkUser marker, @RequestParam Long questionId){
    	List<LevelStatDTO> levelStatDTOs = new ArrayList<>();
        markTaskRepo.countGroupByLevel(questionId,marker.getId())
                .forEach(o -> {
                    levelStatDTOs.add(levelStatAssembler.toDTO(o));
                });
        MarkSubject markSubject = markSubjectRepo.findOne(marker.getWorkId() + "-" + marker.getSubject().toString());
        List<Level> levels = levelRepo.findByWorkId(markSubject.getWorkId());
        for(Level level : levels){
            long count = levelStatDTOs.stream().filter(l -> String.valueOf(l.getId()).equals(level.getCode())).count();
            if(count == 0){
                LevelStatDTO dto = new LevelStatDTO();
                dto.setId(level.getCode());
                dto.setCount(0);
                dto.setPercent(0.0);
                levelStatDTOs.add(dto);
            }
        }
        long total = markTaskRepo.countByMarkerIdAndQuestionId(marker.getId(),questionId);
        levelStatDTOs.forEach(o -> {
            if(o.getId()!=null){
                double p = (double)o.getCount()/total;
                BigDecimal bd = new BigDecimal(p).setScale(3, RoundingMode.HALF_EVEN);
            	o.setPercent(bd.doubleValue());
            }
        });
        return levelStatDTOs;
    }

    /**
     * 各试题统计数据
     * @param marker 评卷员用户id
     * @return
     */
    @RequestMapping(value = "{marker}/stat/questions",method = RequestMethod.GET)
    public List<QuestionStatDTO> questions(@PathVariable MarkUser marker){
        List<QuestionStatDTO> questionStatDTOs = new ArrayList<>();
        List<Object[]> qStats = null;
        MarkSubject markSubject = markSubjectRepo.findOne(marker.getWorkId() + "-" + marker.getSubject().toString());
        qStats = markTaskRepo.countGroupByQuestion(marker.getId(),markSubject.getStage().ordinal());
        if(qStats != null) {
            for (Object[] objects : qStats) {
                QuestionStatDTO dto = questionStatAssembler.toDTO(objects);
                questionStatDTOs.add(dto);
            }
        }
        return questionStatDTOs;
    }
	
}
